@@ -10,7 +10,7 @@ use List::Util qw(first);
 use namespace::autoclean;
 extends 'App::Sqitch::Command';
 
-our $VERSION = '0.940';
+our $VERSION = '0.950';
 
 has to_target => (
     is  => 'ro',
@@ -22,6 +22,13 @@ has no_prompt => (
     isa => 'Bool'
 );
 
+has log_only => (
+    is       => 'ro',
+    isa      => 'Bool',
+    required => 1,
+    default  => 0,
+);
+
 has variables => (
     is       => 'ro',
     isa      => 'HashRef',
@@ -40,6 +47,7 @@ sub options {
     return qw(
         to-target|to|target=s
         set|s=s%
+        log-only
         y
     );
 }
@@ -47,8 +55,10 @@ sub options {
 sub configure {
     my ( $class, $config, $opt ) = @_;
 
-    my %params;
-    $params{to_target} = $opt->{to_target} if exists $opt->{to_target};
+    my %params = map { $_ => $opt->{$_} } grep { exists $opt->{$_} } qw(
+        to_target
+        log_only
+    );
 
     if ( my $vars = $opt->{set} ) {
         # Merge with config.
@@ -72,7 +82,7 @@ sub execute {
     my $engine = $self->sqitch->engine;
     $engine->no_prompt( $self->no_prompt );
     if (my %v = %{ $self->variables }) { $engine->set_variables(%v) }
-    $engine->revert( $self->to_target // shift );
+    $engine->revert( $self->to_target // shift, $self->log_only );
     return $self;
 }
 
@@ -134,7 +144,7 @@ David E. Wheeler <david@justatheory.com>
 
 =head1 License
 
-Copyright (c) 2012 iovation Inc.
+Copyright (c) 2012-2013 iovation Inc.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
